public struct Point: Sendable {
	public let coordinates: [Double]
}

extension Point {
	public func euclideanDistance(to other: Self) async -> Double {
		await squareOfEuclideanDistance(to: other).squareRoot()
	}

	@inlinable
	public func squareOfEuclideanDistance(to other: Self) async -> Double {
		let zippedCoordinates = zip(self.coordinates, other.coordinates)

		return await withTaskGroup(of: Double.self) { group -> Double in
			for coordinatePair in zippedCoordinates {
				group.addTask {
					return squareOfDistance(between: coordinatePair)
				}
			}
			return await group.reduce(0, +)
		}

		@Sendable
		func squareOfDistance(between coordinatePair: (Double, Double)) -> Double {
			let distance = coordinatePair.0 - coordinatePair.1
			return distance * distance
		}
	}
}
